@@ -45,14 +45,13 @@
 #include <sys/stat.h>
 
 #include <ctype.h>
+#include <stdlib.h>
 #include <stdio.h>
 #include <unistd.h>
 #include <fcntl.h>
 #include <syslog.h>
 #include <signal.h>
 #include <errno.h>
-#include <err.h>
-#include <paths.h>
 #include <stdarg.h>
 #include <pwd.h>
 #include <time.h>
@@ -274,8 +273,30 @@ int sp_run(const char* configfile, const char* pidfile, int dbg_level)
     else if(g_state.outname != NULL && g_state.transparent)
         warnx("the " CFG_OUTADDR " option will be ignored when " CFG_TRANSPARENT " is enabled");
 
-    sp_messagex(NULL, LOG_DEBUG, "starting up...");
+    sp_messagex(NULL, LOG_DEBUG, "starting up (%s)...", VERSION);
 
+    /* Create the socket */
+    sock = socket(SANY_TYPE(g_state.listenaddr), SOCK_STREAM, 0);
+    if(sock < 0)
+        err(1, "couldn't open socket");
+
+    fcntl(sock, F_SETFD, fcntl(sock, F_GETFD, 0) | FD_CLOEXEC);    
+    setsockopt(sock, SOL_SOCKET, SO_REUSEADDR, (void *)&true, sizeof(true));
+    
+    /* Unlink the socket file if it exists */
+    if(SANY_TYPE(g_state.listenaddr) == AF_UNIX)
+        unlink(g_state.listenname);
+
+    if(bind(sock, &SANY_ADDR(g_state.listenaddr), SANY_LEN(g_state.listenaddr)) != 0)
+        err(1, "couldn't bind to address: %s", g_state.listenname);
+
+    /* Let 5 connections queue up */
+    if(listen(sock, 5) != 0)
+    {
+        sp_message(NULL, LOG_CRIT, "couldn't listen on socket");
+        exit(1);
+    }
+      	
     /* Drop privileges before daemonizing */
     drop_privileges();
     
@@ -296,34 +317,6 @@ int sp_run(const char* configfile, const char* pidfile, int dbg_level)
         openlog(g_state.name, 0, LOG_MAIL);
     }
 
-    /* Create the socket */
-    sock = socket(SANY_TYPE(g_state.listenaddr), SOCK_STREAM, 0);
-    if(sock < 0)
-    {
-        sp_message(NULL, LOG_CRIT, "couldn't open socket");
-        exit(1);
-    }
-
-    fcntl(sock, F_SETFD, fcntl(sock, F_GETFD, 0) | FD_CLOEXEC);    
-    setsockopt(sock, SOL_SOCKET, SO_REUSEADDR, (void *)&true, sizeof(true));
-    
-    /* Unlink the socket file if it exists */
-    if(SANY_TYPE(g_state.listenaddr) == AF_UNIX)
-        unlink(g_state.listenname);
-
-    if(bind(sock, &SANY_ADDR(g_state.listenaddr), SANY_LEN(g_state.listenaddr)) != 0)
-    {
-        sp_message(NULL, LOG_CRIT, "couldn't bind to address: %s", g_state.listenname);
-        exit(1);
-    }
-
-    /* Let 5 connections queue up */
-    if(listen(sock, 5) != 0)
-    {
-        sp_message(NULL, LOG_CRIT, "couldn't listen on socket");
-        exit(1);
-    }
-      
     sp_messagex(NULL, LOG_DEBUG, "created socket: %s", g_state.listenname);
 
     /* Handle some signals */
@@ -493,7 +486,7 @@ static void connection_loop(int sock)
         /* Set timeouts on client */
         if(setsockopt(fd, SOL_SOCKET, SO_RCVTIMEO, &(g_state.timeout), sizeof(g_state.timeout)) < 0 ||
            setsockopt(fd, SOL_SOCKET, SO_SNDTIMEO, &(g_state.timeout), sizeof(g_state.timeout)) < 0)
-            sp_message(NULL, LOG_WARNING, "couldn't set timeouts on incoming connection");        
+            sp_message(NULL, LOG_DEBUG, "couldn't set timeouts on incoming connection");        
 
         fcntl(fd, F_SETFD, fcntl(fd, F_GETFD, 0) | FD_CLOEXEC);    
 
@@ -1328,7 +1321,6 @@ static void make_date(spctx_t* ctx, char* date)
 {
     size_t date_len;
     struct tm t2;
-    long gmt;
     time_t t;
 
     /* Get a basic date like: 'Wed Jun 30 21:49:08 1993' */    
@@ -1341,14 +1333,50 @@ static void make_date(spctx_t* ctx, char* date)
         return;
     }
       
-    /* Now add the TZ */
     trim_end(date);
-    gmt = t2.tm_gmtoff;
     date_len = strlen(date);
-      
-    snprintf(date + date_len, MAX_DATE_LENGTH - date_len, " %+03d%02d (%s)", 
-             (int)(gmt / 3600), (int)(gmt % 3600), t2.tm_zone);
 
+    {
+#ifdef HAVE_TM_GMTOFF
+        time_t timezone = t2.tm_gmtoff;
+        const char *tzname[2] = { t2.tm_zone, t2.tm_zone };
+
+        snprintf(date + date_len, MAX_DATE_LENGTH - date_len, " %+03d%02d (%s)", 
+                 (int)(timezone / 3600), (int)(timezone % 3600),
+                 tzname[t2.tm_isdst ? 1 : 0]);
+#else
+        /* Apparently Solaris needs this nasty hack.... */
+        #define DAY_MIN         (24 * HOUR_MIN)
+        #define HOUR_MIN        60
+        #define MIN_SEC         60
+
+        struct tm gmt;
+        struct tm *lt;
+        int off;
+
+        gmt = *gmtime(&t);
+        lt = localtime(&t);
+        off = (lt->tm_hour - gmt.tm_hour) * HOUR_MIN + lt->tm_min - gmt.tm_min;
+
+        if (lt->tm_year < gmt.tm_year)
+            off -= DAY_MIN;
+        else if (lt->tm_year > gmt.tm_year)
+            off += DAY_MIN;
+        else if (lt->tm_yday < gmt.tm_yday)
+            off -= DAY_MIN;
+        else if (lt->tm_yday > gmt.tm_yday)
+            off += DAY_MIN;
+        if (lt->tm_sec <= gmt.tm_sec - MIN_SEC)
+            off -= 1;
+        else if (lt->tm_sec >= gmt.tm_sec + MIN_SEC)
+            off += 1;
+
+        snprintf(date + date_len, MAX_DATE_LENGTH - date_len,
+                 " %+03d%02d (%s)", (int)(off / HOUR_MIN), (int)(abs(off) % HOUR_MIN),
+                 tzname[lt->tm_isdst ? 1 : 0]);
+#endif    
+    }
+    
     /* Break it off just in case */
     date[MAX_DATE_LENGTH - 1] = 0;
 }
@@ -1482,7 +1510,7 @@ int sp_done_data(spctx_t* ctx)
             
         sp_messagex(ctx, LOG_DEBUG, "server refused data transfer");
             
-        RETURN(-1);
+        RETURN(0);
     }
 
     sp_messagex(ctx, LOG_DEBUG, "sending from cache file: %s", ctx->cachename);
